from sugar.activity.activity import Activity, ActivityToolbox
from sugar.datastore.datastore import DSMetadata

import gtk
import gtk.gdk
import gobject
from gettext import gettext

import clock_innards

SERVICE = "org.laptop.ClockActivity"
IFACE = SERVICE
PATH = "/org/laptop/ClockActivity"

class ClockActivity(Activity):
    def __init__(self, handle):
        Activity.__init__(self, handle, create_jobject=False)
        gobject.threads_init()
        name = gettext('Clock')
        self.set_title(name)
        self._metadata = DSMetadata()
        self._metadata['title'] = name

        self.close = self.destroy

        toolbox = ActivityToolbox(self)
        self.set_toolbox(toolbox)
        toolbox.show()
        
        self._innards = clock_innards.Clock()
        self.set_canvas(self._innards.display)

        self.connect("notify::active", self._active_cb)
        self.add_events(gtk.gdk.KEY_RELEASE_MASK)
        self.connect("key-release-event", self._keypress_cb)
        self.add_events(gtk.gdk.VISIBILITY_NOTIFY_MASK)
        self.connect("visibility-notify-event", self._visible_cb)

        self.show_all()

        #Dirty Hide like Terminal
        toolbar = toolbox.get_activity_toolbar()
        toolbar.share.hide()
        toolbar.keep.hide()

        self._innards.start()
        
    def _active_cb(self, widget, event):
        if self.props.active:
            self._innards.resume()
        else:
            self._innards.pause()
            
    def _keypress_cb(self, widget, event):
        name = gtk.gdk.keyval_name(event.keyval)
        if name == 'Escape':
            self.destroy()
    
    def _visible_cb(self, widget, event):
        if event.state == gtk.gdk.VISIBILITY_FULLY_OBSCURED:
            self._innards.pause()
        else:
            self._innards.resume()
    
    def get_metadata(self):
        return self._metadata
