import gtk
import gtk.gdk
import time
import cairo
import thread
import threading
import pango
import math
import gobject
import gc

class Clock():
    _old_date = None
    
    def __init__(self):
        self._prepare_display()
        self._ev = threading.Event()
        self._ev.set()
    
    def start(self):
        cmap = self.analog.get_colormap()
        v = 65535
        self._base_gc = self.analog.window.new_gc(background=cmap.alloc_color(v,v,v))
        self._hour_gc = self.analog.window.new_gc(foreground=cmap.alloc_color(0,0,0), line_width=8)
        self._minute_gc = self.analog.window.new_gc(foreground=cmap.alloc_color(0,0,v), line_width=4)
        self._second_gc = self.analog.window.new_gc(foreground=cmap.alloc_color(v,0,0), line_width=2)
        
        self._backmap = gtk.gdk.Pixmap(self.analog.window, self._cwidth, self._cheight)
        self._backmap.draw_pixbuf(self._base_gc, self._clockface,0,0,0,0)
        self._tempmap = gtk.gdk.Pixmap(self.analog.window, self._cwidth, self._cheight)

        thread.start_new_thread(self._update, ())
        del(self._clockface)
        gc.collect()
    
    def pause(self):
        self._ev.clear()
        
    def resume(self):
        self._ev.set()
        
    def _prepare_display(self):
        self.display = gtk.VBox()
        
        self._clockface = gtk.gdk.pixbuf_new_from_file('clock.svg')
        
        self.analog = gtk.DrawingArea()
        self._cwidth = self._clockface.get_width()
        self._cheight = self._clockface.get_height()
        self.analog.set_size_request(self._cwidth, self._cheight)
        
        squeezer = gtk.HBox()
        squeezer.pack_start(self.analog, expand=True, fill=False)
        self.display.pack_start(squeezer, expand=True, fill=False, padding=6)
        
        self.digital = gtk.Label()
                
        digifont = pango.FontDescription()
        digifont.set_family("monospace")
        digifont.set_absolute_size(200*pango.SCALE)
        
        self.digital.modify_font(digifont)
        self.digital.set_single_line_mode(True)
        
        eb = gtk.EventBox()
        eb.add(self.digital)
        eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        
        self.date = gtk.Label()
        
        datefont = pango.FontDescription()
        datefont.set_family("sans")
        datefont.set_absolute_size(100*pango.SCALE)
        
        self.date.modify_font(datefont)
        self.date.set_single_line_mode(True)
        
        self.display.pack_end(self.date)
        self.display.pack_end(eb)
    
    def _update(self):
        evd = threading.Event()
        eva = threading.Event()
        evx = threading.Event()
        while True:
            self._ev.wait()
            t = time.time()
            lt = time.localtime(t)
            #gobject.threads_enter()
            evd.clear()
            eva.clear()
            gobject.idle_add(self._set_clockface, lt, eva)
            gobject.idle_add(self._set_digiclock, lt, evd)
            gobject.idle_add(self._set_date, lt, evx)
            evd.wait()
            eva.wait()
            evx.wait()
            #gobject.threads_leave()
            self._sleep_until(math.floor(t) + 1.2)
    
    def _sleep_until(self, t):
        time.sleep(max(0, t - time.time()))

    def _set_date(self, t, ev):
        newdate = time.strftime('%x', t)
        if not (self._old_date == newdate):
            self.date.set_text(newdate)
            self._old_date = newdate
        ev.set()
        return False
    
    def _set_digiclock(self, t, ev):
        h = time.strftime('%H',t)
        m = time.strftime('%M',t)
        s = time.strftime('%S',t)
        #self.digital.set_text(time.strftime('%H:%M:%S',t))
        self.digital.set_markup(h + ":<span foreground=\"blue\" >" + m + "</span>:<span foreground=\"red\">" + s + "</span>")
        ev.set()
        return False

    def _set_clockface(self, t, ev):
        
        x0 = self._cwidth/2
        y0 = self._cheight/2
        
        second = t.tm_sec
        s_angle = second * math.pi / 30
        s_r = self._cwidth * 0.5 * 0.9
        
        minute = t.tm_min
        m_angle = (s_angle / 60) + minute * math.pi / 30
        m_r = self._cwidth * 0.5 * 0.8
        
        hour = t.tm_hour % 12
        h_angle = (m_angle / 12) + hour * math.pi / 6
        h_r = self._cwidth * 0.5 * 0.7
        
        self._tempmap.draw_drawable(self._base_gc, self._backmap,0,0,0,0,-1,-1)
        self._tempmap.draw_line(self._hour_gc, x0, y0, x0 + int(h_r*math.sin(h_angle)), y0 - int(h_r*math.cos(h_angle)))
        self._tempmap.draw_line(self._minute_gc, x0, y0, x0 + int(m_r*math.sin(m_angle)), y0 - int(m_r*math.cos(m_angle)))
        self._tempmap.draw_line(self._second_gc, x0, y0, x0 + int(s_r*math.sin(s_angle)), y0 - int(s_r*math.cos(s_angle)))
        
        self.analog.window.draw_drawable(self._base_gc, self._tempmap,0,0,0,0,-1,-1)
        ev.set()
        return False
        
        
        

